import styles from "./DosHeader.module.scss";
import { Flex } from "antd";
import React, { useContext, useEffect, useRef, useState } from "react";
import type { GetRef, InputRef, TableProps } from "antd";
import { Form, Input, Table } from "antd";
import { invoke } from "@tauri-apps/api/core";
import { cloneDeep } from "lodash-es";

type FormInstance<T> = GetRef<typeof Form<T>>;

const EditableContext = React.createContext<FormInstance<any> | null>(null);

interface DosHeaderFieldProps {
  name: string;
  value: number | [number];
  size: number;
  offset: number;
  field_type: string;
}

interface DosHeaderResponse {
  image_dos_header: DosHeaderFieldResponse;
  base_offset: number;
}

interface DosHeaderFieldResponse {
  e_magic: number;
  e_cblp: number;
  e_cp: number;
  e_crlc: number;
  e_cparhdr: number;
  e_minalloc: number;
  e_maxalloc: number;
  e_ss: number;
  e_sp: number;
  e_csum: number;
  e_ip: number;
  e_cs: number;
  e_lfarlc: number;
  e_ovno: number;
  e_res: number;
  e_oemid: number;
  e_oeminfo: number;
  e_res2: number;
  e_lfanew: number;
}

// 定义DOS Header的数据结构
interface IDosHeader {
  e_magic: DosHeaderFieldProps;
  e_cblp: DosHeaderFieldProps;
  e_cp: DosHeaderFieldProps;
  e_crlc: DosHeaderFieldProps;
  e_cparhdr: DosHeaderFieldProps;
  e_minalloc: DosHeaderFieldProps;
  e_maxalloc: DosHeaderFieldProps;
  e_ss: DosHeaderFieldProps;
  e_sp: DosHeaderFieldProps;
  e_csum: DosHeaderFieldProps;
  e_ip: DosHeaderFieldProps;
  e_cs: DosHeaderFieldProps;
  e_lfarlc: DosHeaderFieldProps;
  e_ovno: DosHeaderFieldProps;
  e_res: DosHeaderFieldProps;
  e_oemid: DosHeaderFieldProps;
  e_oeminfo: DosHeaderFieldProps;
  e_res2: DosHeaderFieldProps;
  e_lfanew: DosHeaderFieldProps;
}

const templateData: IDosHeader = {
  e_magic: {
    name: "e_magic",
    value: 0,
    size: 2,
    offset: 0,
    field_type: "WORD",
  },
  e_cblp: {
    name: "e_cblp",
    value: 0,
    size: 2,
    offset: 2,
    field_type: "WORD",
  },
  e_cp: {
    name: "e_cp",
    value: 0,
    size: 2,
    offset: 4,
    field_type: "WORD",
  },
  e_crlc: {
    name: "e_crlc",
    value: 0,
    size: 2,
    offset: 6,
    field_type: "WORD",
  },
  e_cparhdr: {
    name: "e_cparhdr",
    value: 0,
    size: 2,
    offset: 8,
    field_type: "WORD",
  },
  e_minalloc: {
    name: "e_minalloc",
    value: 0,
    size: 2,
    offset: 10,
    field_type: "WORD",
  },
  e_maxalloc: {
    name: "e_maxalloc",
    value: 0,
    size: 2,
    offset: 12,
    field_type: "WORD",
  },
  e_ss: {
    name: "e_ss",
    value: 0,
    size: 2,
    offset: 14,
    field_type: "WORD",
  },
  e_sp: {
    name: "e_sp",
    value: 0,
    size: 2,
    offset: 16,
    field_type: "WORD",
  },
  e_csum: {
    name: "e_csum",
    value: 0,
    size: 2,
    offset: 18,
    field_type: "WORD",
  },
  e_ip: {
    name: "e_ip",
    value: 0,
    size: 2,
    offset: 20,
    field_type: "WORD",
  },
  e_cs: {
    name: "e_cs",
    value: 0,
    size: 2,
    offset: 22,
    field_type: "WORD",
  },
  e_lfarlc: {
    name: "e_lfarlc",
    value: 0,
    size: 2,
    offset: 24,
    field_type: "WORD",
  },
  e_ovno: {
    name: "e_ovno",
    value: 0,
    size: 2,
    offset: 26,
    field_type: "WORD",
  },
  e_res: {
    name: "e_res",
    value: null,
    size: 8,
    offset: 28,
    field_type: "[WORD; 4]",
  },
  e_oemid: {
    name: "e_oemid",
    value: 0,
    size: 2,
    offset: 36,
    field_type: "WORD",
  },
  e_oeminfo: {
    name: "e_oeminfo",
    value: 0,
    size: 2,
    offset: 38,
    field_type: "WORD",
  },
  e_res2: {
    name: "e_res2",
    value: 0,
    size: 20,
    offset: 40,
    field_type: "[WORD; 10]",
  },
  e_lfanew: {
    name: "e_lfanew",
    value: 0,
    size: 4,
    offset: 60,
    field_type: "DWORD",
  },
};

interface Item {
  name: string; // 字段名称
  offset: string; // 偏移
  size: Number; // 字段大小
  value: string; // 字段值
  children?: Item[];
}

interface EditableRowProps {
  index: number;
}

const EditableRow: React.FC<EditableRowProps> = ({ index, ...props }) => {
  const [form] = Form.useForm();
  return (
    <Form form={form} component={false}>
      <EditableContext.Provider value={form}>
        <tr {...props} />
      </EditableContext.Provider>
    </Form>
  );
};

interface EditableCellProps {
  title: React.ReactNode;
  editable: boolean;
  dataIndex: keyof Item;
  record: Item;
  handleSave: (record: Item) => void;
}

const EditableCell: React.FC<React.PropsWithChildren<EditableCellProps>> = ({
  title,
  editable,
  children,
  dataIndex,
  record,
  handleSave,
  ...restProps
}) => {
  const [editing, setEditing] = useState(false);
  const inputRef = useRef<InputRef>(null);
  const form = useContext(EditableContext)!;

  useEffect(() => {
    if (editing) {
      inputRef.current?.focus();
    }
  }, [editing]);

  const toggleEdit = () => {
    setEditing(!editing);
    form.setFieldsValue({ [dataIndex]: record[dataIndex] });
  };

  const save = async () => {
    try {
      const values = await form.validateFields();
      toggleEdit();
      handleSave({ ...record, ...values });
    } catch (errInfo) {
      console.log("Save failed:", errInfo);
    }
  };

  let childNode = children;

  if (editable) {
    childNode = editing ? (
      <Form.Item
        style={{ margin: 0 }}
        name={dataIndex}
        rules={[{ required: true, message: `${title} is required.` }]}
      >
        <Input size="small" ref={inputRef} onPressEnter={save} onBlur={save} />
      </Form.Item>
    ) : (
      <div
        className="editable-cell-value-wrap"
        style={{ paddingInlineEnd: 24 }}
        onClick={toggleEdit}
      >
        {children}
      </div>
    );
  }

  return <td {...restProps}>{childNode}</td>;
};

interface DataType {
  name: string; // 字段名称
  offset: string; // 偏移
  size: string | Number; // 字段大小
  value: string; // 字段值
  field_type: string; // 字段类型
  children?: DataType[];
}

type ColumnTypes = Exclude<TableProps<DataType>["columns"], undefined>;

const App = () => {
  const [offsetHex, setOffsetHex] = useState(false);
  const [dataSource, setDataSource] = useState<DataType[]>([]);

  useEffect(() => {
    invoke("command_get_pe_data_dos_header").then(
      (resData: DosHeaderResponse) => {
        console.log("resData", resData);
        // 格式化数据
        let data = cloneDeep(templateData); // 先clone一个模板
        let base_offset = resData.base_offset;
        Object.keys(resData.image_dos_header).forEach((key) => {
          let value = resData.image_dos_header[key];
          data[key].offset = base_offset + data[key].offset;
          if (Array.isArray(value)) {
            // 如果是数组
            data[key].children = value.map((item, index) => {
              let child_size = data[key].size / value.length;
              const extractWord = (input) => {
                const match = input.match(/\[([\w]+);/);
                return match ? match[1] : null; // 如果匹配到，返回结果；否则返回 null
              };
              return {
                name: `${key}[${index}]`,
                offset: index * child_size + data[key].offset,
                size: child_size,
                value: "0x" + item.toString(16).padStart(4, "0").toUpperCase(),
                field_type: extractWord(data[key].field_type),
              };
            });
          } else {
            data[key].value =
              "0x" + value.toString(16).padStart(4, "0").toUpperCase();
          }
        });
        setDataSource(Object.values(data));
      }
    );
  }, []);
  
  const defaultColumns = [
    {
      title: "字段名称",
      dataIndex: "name",
      width: "30%",
    },
    {
      title: "偏移量",
      dataIndex: "offset",
      defaultSortOrder: "ascend",
      render: (text) => {
        return offsetHex ? `0x${text.toString(16).toUpperCase()}` : text;
      },
      onHeaderCell: (column) => ({
        onClick: () => {
          setOffsetHex(!offsetHex);
        }
      })
    },
    {
      title: "字段类型",
      dataIndex: "field_type",
    },
    {
      title: "字段大小",
      dataIndex: "size",
    },
    {
      title: "字段值",
      dataIndex: "value",
      editable: true,
    },
  ];

  const replaceNodeByName = (
    data: DataType[],
    name: string,
    newNode: DataType
  ): DataType[] => {
    return data.map((item) => {
      if (item.name === name) {
        return newNode;
      }
      if (item.children) {
        return {
          ...item,
          children: replaceNodeByName(item.children, name, newNode),
        };
      }
      return item;
    });
  };

  const handleSave = (row: DataType) => {
    let newData = [...dataSource];
    newData = replaceNodeByName(newData, row.name, row);
    setDataSource(newData);
  };

  const components = {
    body: {
      row: EditableRow,
      cell: EditableCell,
    },
  };

  const columns = defaultColumns.map((col) => {
    if (!col.editable) {
      return col;
    }
    return {
      ...col,
      onCell: (record: DataType) => ({
        record,
        editable: col.editable,
        dataIndex: col.dataIndex,
        title: col.title,
        handleSave,
      }),
    };
  });

  return (
    <Table<DataType>
      components={components}
      rowClassName={() => "editable-row"}
      bordered
      rowKey={(record) => record.name}
      style={{ width: "100%" }}
      dataSource={dataSource}
      columns={columns as ColumnTypes}
      pagination={false}
      size="small"
      sticky={{ offsetHeader: -16 }}
      
    />
  );
};

export default function DosHeader() {
  return (
    <Flex className={styles.root}>
      <App />
    </Flex>
  );
}
