import { Flex, Form, FormInstance, Input, Table } from "antd";
import styles from "./FileHeader.module.scss";
import { useContext, useEffect, useRef, useState } from "react";
import { invoke } from "@tauri-apps/api/core";
import React from "react";
import { cloneDeep } from "lodash-es";
const EditableContext = React.createContext<FormInstance<any> | null>(null);

const templateData = {
  machine: {
    name: "Machine",
    offset: 0,
    size: 2,
    value: null,
    field_type: "WORD",
  },
  number_of_sections: {
    name: "Number of Sections",
    offset: 2,
    size: 2,
    value: null,
    field_type: "WORD",
  },
  time_date_stamp: {
    name: "Time Date Stamp",
    offset: 4,
    size: 4,
    value: null,
    field_type: "DWORD",
  },
  pointer_to_symbol_table: {
    name: "Pointer to Symbol Table",
    offset: 8,
    size: 4,
    value: null,
    field_type: "DWORD",
  },
  number_of_symbols: {
    name: "Number of Symbols",
    offset: 12,
    size: 4,
    value: null,
    field_type: "DWORD",
  },
  size_of_optional_header: {
    name: "Size of Optional Header",
    offset: 16,
    size: 2,
    value: null,
    field_type: "WORD",
  },
  characteristics: {
    name: "Characteristics",
    offset: 18,
    size: 2,
    value: null,
    field_type: "WORD",
  },
};

const EditableRow = ({ index, ...props }) => {
  const [form] = Form.useForm();
  return (
    <Form form={form} component={false}>
      <EditableContext.Provider value={form}>
        <tr {...props} />
      </EditableContext.Provider>
    </Form>
  );
};

const EditableCell = ({
  title,
  editable,
  children,
  dataIndex,
  record,
  handleSave,
  ...restProps
}) => {
  const [editing, setEditing] = useState(false);
  const inputRef = useRef(null);
  const form = useContext(EditableContext)!;

  useEffect(() => {
    if (editing) {
      inputRef.current?.focus();
    }
  }, [editing]);

  const toggleEdit = () => {
    setEditing(!editing);
    form.setFieldsValue({ [dataIndex]: record[dataIndex] });
  };

  const save = async () => {
    try {
      const values = await form.validateFields();
      toggleEdit();
      handleSave({ ...record, ...values });
    } catch (errInfo) {
      console.log("Save failed:", errInfo);
    }
  };

  let childNode = children;

  if (editable) {
    childNode = editing ? (
      <Form.Item
        style={{ margin: 0 }}
        name={dataIndex}
        rules={[{ required: true, message: `${title} is required.` }]}
      >
        <Input size="small" ref={inputRef} onPressEnter={save} onBlur={save} />
      </Form.Item>
    ) : (
      <div
        className="editable-cell-value-wrap"
        style={{ paddingInlineEnd: 24 }}
        onClick={toggleEdit}
      >
        {children}
      </div>
    );
  }

  return <td {...restProps}>{childNode}</td>;
};

const App = () => {
  const [offsetHex, setOffsetHex] = useState(false);
  const [dataSource, setDataSource] = useState([]);
  useEffect(() => {
    invoke("command_get_pe_data_file_header").then((resData: any) => {
      console.log("resData", resData);
      let data = cloneDeep(templateData);
      let base_offset = resData.base_offset;
      Object.keys(data).forEach((key) => {
        data[key].value = resData.file_header[key];
        data[key].offset = base_offset + data[key].offset;
      });
      setDataSource(Object.values(data));
    });
  }, []);

  const defaultColumns = [
    {
      title: "字段名称",
      dataIndex: "name",
      width: "30%",
    },
    {
      title: "偏移量",
      dataIndex: "offset",
      defaultSortOrder: "ascend",
      render: (text) => {
        return offsetHex ? `0x${text.toString(16).toUpperCase()}` : text;
      },
      onHeaderCell: (column) => ({
        onClick: () => {
          setOffsetHex(!offsetHex);
        },
      }),
    },
    {
      title: "字段类型",
      dataIndex: "field_type",
    },
    {
      title: "字段大小",
      dataIndex: "size",
    },
    {
      title: "字段值",
      dataIndex: "value",
      editable: true,
    },
  ];

  const replaceNodeByName = (data, name, newNode) => {
    return data.map((item) => {
      if (item.name === name) {
        return newNode;
      }
      if (item.children) {
        return {
          ...item,
          children: replaceNodeByName(item.children, name, newNode),
        };
      }
      return item;
    });
  };

  const handleSave = (row) => {
    let newData = [...dataSource];
    newData = replaceNodeByName(newData, row.name, row);
    setDataSource(newData);
  };

  const components = {
    body: {
      row: EditableRow,
      cell: EditableCell,
    },
  };

  const columns = defaultColumns.map((col) => {
    if (!col.editable) {
      return col;
    }
    return {
      ...col,
      onCell: (record) => ({
        record,
        editable: col.editable,
        dataIndex: col.dataIndex,
        title: col.title,
        handleSave,
      }),
    };
  });

  return (
    <Table
      components={components}
      rowClassName={() => "editable-row"}
      bordered
      rowKey={(record) => record.name}
      style={{ width: "100%" }}
      dataSource={dataSource}
      columns={columns as any}
      pagination={false}
      size="small"
      sticky={{ offsetHeader: -16 }}
    />
  );
};

export default function FileHeader() {
  return (
    <Flex className={styles.root}>
      <App />
    </Flex>
  );
}
